package com.jiuqi.util.json;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.jiuqi.util.json.JSONArray;
import com.jiuqi.util.json.JSONException;
import com.jiuqi.util.json.JSONObject;

public class JSONTokener {
  private int myIndex;
  
  private String mySource;
  
  public JSONTokener(String s) {
    this.myIndex = 0;
    this.mySource = s;
  }
  
  public void back() {
    if (this.myIndex > 0)
      this.myIndex--; 
  }
  
  public static int dehexchar(char c) {
    if (c >= '0' && c <= '9')
      return c - 48; 
    if (c >= 'A' && c <= 'F')
      return c - 55; 
    if (c >= 'a' && c <= 'f')
      return c - 87; 
    return -1;
  }
  
  public boolean more() {
    return (this.myIndex < this.mySource.length());
  }
  
  public char next() {
    if (more()) {
      char c = this.mySource.charAt(this.myIndex);
      this.myIndex++;
      return c;
    } 
    return Character.MIN_VALUE;
  }
  
  public char next(char c) throws JSONException {
    char n = next();
    if (n != c)
      throw syntaxError("Expected '" + c + "' and instead saw '" + 
          n + "'"); 
    return n;
  }
  
  public String next(int n) throws JSONException {
    int i = this.myIndex;
    int j = i + n;
    if (j >= this.mySource.length())
      throw syntaxError("Substring bounds error"); 
    this.myIndex += n;
    return this.mySource.substring(i, j);
  }
  
  public char nextClean() throws JSONException {
    char c;
    label38: do {
      c = next();
      if (c == '/') {
        switch (next()) {
          case '/':
            while (true) {
              c = next();
              if (c != '\n') {
                if (c != '\r') {
                  if (c == '\000')
                    continue label38; 
                  continue;
                } 
                continue label38;
              } 
              continue label38;
            } 
          case '*':
            while (true) {
              c = next();
              if (c == '\000')
                throw syntaxError("Unclosed comment"); 
              if (c == '*') {
                if (next() == '/')
                  continue label38; 
                back();
              } 
            } 
        } 
        back();
        return '/';
      } 
      if (c != '#')
        continue; 
      while (true) {
        c = next();
        if (c != '\n') {
          if (c != '\r') {
            if (c == '\000')
              continue label38; 
            continue;
          } 
          continue label38;
        } 
        continue label38;
      } 
    } while (c != '\000' && c <= ' ');
    return c;
  }
  
  public String nextString(char quote) throws JSONException {
    StringBuffer sb = new StringBuffer();
    while (true) {
      char c = next();
      switch (c) {
        case '\000':
        case '\n':
        case '\r':
          throw syntaxError("Unterminated string");
        case '\\':
          c = next();
          switch (c) {
            case 'b':
              sb.append('\b');
              continue;
            case 't':
              sb.append('\t');
              continue;
            case 'n':
              sb.append('\n');
              continue;
            case 'f':
              sb.append('\f');
              continue;
            case 'r':
              sb.append('\r');
              continue;
            case 'u':
              sb.append((char)Integer.parseInt(next(4), 16));
              continue;
            case 'x':
              sb.append((char)Integer.parseInt(next(2), 16));
              continue;
          } 
          sb.append(c);
          continue;
      } 
      if (c == quote)
        return sb.toString(); 
      sb.append(c);
    } 
  }
  
  public String nextTo(char d) {
    StringBuffer sb = new StringBuffer();
    while (true) {
      char c = next();
      if (c == d || c == '\000' || c == '\n' || c == '\r') {
        if (c != '\000')
          back(); 
        return sb.toString().trim();
      } 
      sb.append(c);
    } 
  }
  
  public String nextTo(String delimiters) {
    StringBuffer sb = new StringBuffer();
    while (true) {
      char c = next();
      if (delimiters.indexOf(c) >= 0 || c == '\000' || 
        c == '\n' || c == '\r') {
        if (c != '\000')
          back(); 
        return sb.toString().trim();
      } 
      sb.append(c);
    } 
  }
  
  public Object nextValue() throws JSONException {
    char c = nextClean();
    switch (c) {
      case '"':
      case '\'':
        return nextString(c);
      case '{':
        back();
        return new JSONObject(this);
      case '(':
      case '[':
        back();
        return new JSONArray(this);
    } 
    StringBuffer sb = new StringBuffer();
    char b = c;
    while (c >= ' ' && ",:]}/\\\"[{;=#".indexOf(c) < 0) {
      sb.append(c);
      c = next();
    } 
    back();
    String s = sb.toString().trim();
    if (s.equals(""))
      throw syntaxError("Missing value"); 
    if (s.equalsIgnoreCase("true"))
      return Boolean.TRUE; 
    if (s.equalsIgnoreCase("false"))
      return Boolean.FALSE; 
    if (s.equalsIgnoreCase("null"))
      return JSONObject.NULL; 
    if ((b >= '0' && b <= '9') || b == '.' || b == '-' || b == '+') {
      if (b == '0')
        if (s.length() > 2 && (
          s.charAt(1) == 'x' || s.charAt(1) == 'X')) {
          try {
            return new Integer(Integer.parseInt(s.substring(2), 
                  16));
          } catch (Exception exception) {}
        } else {
          try {
            return new Integer(Integer.parseInt(s, 8));
          } catch (Exception exception) {}
        }  
      try {
        return new Integer(s);
      } catch (Exception exception) {
        try {
          return new Long(s);
        } catch (Exception exception1) {
          try {
            return new Double(s);
          } catch (Exception exception2) {
            return s;
          } 
        } 
      } 
    } 
    return s;
  }
  
  public char skipTo(char to) {
    int index = this.myIndex;
    while (true) {
      char c = next();
      if (c == '\000') {
        this.myIndex = index;
        return c;
      } 
      if (c == to) {
        back();
        return c;
      } 
    } 
  }
  
  public boolean skipPast(String to) {
    this.myIndex = this.mySource.indexOf(to, this.myIndex);
    if (this.myIndex < 0) {
      this.myIndex = this.mySource.length();
      return false;
    } 
    this.myIndex += to.length();
    return true;
  }
  
  public JSONException syntaxError(String message) {
    return new JSONException(String.valueOf(message) + toString());
  }
  
  public String toString() {
    return " at character " + this.myIndex + " of " + this.mySource;
  }
}
